@@ -42,6 +42,7 @@
 #include <cassert>
 #include <cstdint>
 #include <map>
+#include <unordered_map>
 
 namespace SPIRV {
 
@@ -71,8 +72,9 @@ public:
 
   TranslatorOpts() = default;
 
-  TranslatorOpts(VersionNumber Max, const ExtensionsStatusMap &Map = {})
-      : MaxVersion(Max), ExtStatusMap(Map) {}
+  TranslatorOpts(VersionNumber Max, const ExtensionsStatusMap &Map = {},
+                 bool ArgNameMD = false)
+      : MaxVersion(Max), ExtStatusMap(Map), GenKernelArgNameMD(ArgNameMD) {}
 
   bool isAllowedToUseVersion(VersionNumber RequestedVersion) const {
     return RequestedVersion <= MaxVersion;
@@ -88,15 +90,35 @@ public:
 
   VersionNumber getMaxVersion() const { return MaxVersion; }
 
+  bool isGenArgNameMDEnabled() const { return GenKernelArgNameMD; }
+
   void enableAllExtensions() {
 #define EXT(X) ExtStatusMap[ExtensionID::X] = true;
 #include "LLVMSPIRVExtensions.inc"
 #undef EXT
   }
 
+  void enableGenArgNameMD() { GenKernelArgNameMD = true; }
+
+  void setSpecConst(uint32_t SpecId, uint64_t SpecValue) {
+    ExternalSpecialization[SpecId] = SpecValue;
+  }
+
+  bool getSpecializationConstant(uint32_t SpecId, uint64_t &Value) const {
+    auto It = ExternalSpecialization.find(SpecId);
+    if (It == ExternalSpecialization.end())
+      return false;
+    Value = It->second;
+    return true;
+  }
+
 private:
+  // Common translation options
   VersionNumber MaxVersion = VersionNumber::MaximumVersion;
   ExtensionsStatusMap ExtStatusMap;
+  // SPIR-V to LLVM translation options
+  bool GenKernelArgNameMD;
+  std::unordered_map<uint32_t, uint64_t> ExternalSpecialization;
 };
 
 } // namespace SPIRV
