@@ -165,6 +165,20 @@ void LLVMToSPIRVDbgTran::transLocationInfo() {
             // SPIR-V doesn't support llvm.dbg.label intrinsic translation
             continue;
           }
+          if (II->getIntrinsicID() == Intrinsic::annotation ||
+              II->getIntrinsicID() == Intrinsic::var_annotation ||
+              II->getIntrinsicID() == Intrinsic::ptr_annotation) {
+            // llvm call instruction for llvm .*annotation intrinsics
+            // is translated into SPIR-V instruction only if it represents
+            // call of __builtin_intel_fpga_reg() builtin. In other cases this
+            // instruction is dropped. In these cases debug info for this call
+            // should be skipped too.
+            // TODO: Remove skipping of debug info when *.annotation call will
+            //       be handled in a better way during SPIR-V translation.
+            V = SPIRVWriter->getTranslatedValue(&I);
+            if (!V || V->getOpCode() != OpFPGARegINTEL)
+              continue;
+          }
         }
         const DebugLoc &DL = I.getDebugLoc();
         if (!DL.get()) {
@@ -191,6 +205,15 @@ void LLVMToSPIRVDbgTran::transLocationInfo() {
           LineNo = DL.getLine();
           Col = DL.getCol();
           V = SPIRVWriter->getTranslatedValue(&I);
+          // According to the spec, OpLine for an OpBranch/OpBranchConditional
+          // must precede the merge instruction and not the branch instruction
+          auto *VPrev = static_cast<SPIRVInstruction *>(V)->getPrevious();
+          if (VPrev && (VPrev->getOpCode() == OpLoopMerge ||
+                        VPrev->getOpCode() == OpLoopControlINTEL)) {
+            assert(V->getOpCode() == OpBranch ||
+                   V->getOpCode() == OpBranchConditional);
+            V = VPrev;
+          }
           BM->addLine(V, File ? File->getId() : getDebugInfoNone()->getId(),
                       LineNo, Col);
         }
@@ -611,7 +634,6 @@ LLVMToSPIRVDbgTran::transDbgCompositeType(const DICompositeType *CT) {
 
   SPIRVEntry *Res =
       BM->addDebugInfo(SPIRVDebug::TypeComposite, getVoidTy(), Ops);
-  BM->replaceForward(Tmp, Res);
 
   // Translate template parameters.
   if (DITemplateParameterArray TP = CT->getTemplateParams()) {
@@ -623,6 +645,7 @@ LLVMToSPIRVDbgTran::transDbgCompositeType(const DICompositeType *CT) {
     }
     Res = BM->addDebugInfo(SPIRVDebug::TypeTemplate, getVoidTy(), Args);
   }
+  BM->replaceForward(Tmp, Res);
   MDMap[CT] = Res;
   return Res;
 }
