@@ -358,7 +358,7 @@ template <typename T> std::string getFullPath(const T *Scope) {
   if (sys::path::is_absolute(Filename))
     return Filename;
   SmallString<16> DirName = Scope->getDirectory();
-  sys::path::append(DirName, Filename);
+  sys::path::append(DirName, sys::path::Style::posix, Filename);
   return DirName.str().str();
 }
 
@@ -416,7 +416,7 @@ bool isPipeStorageInitializer(Instruction *Inst);
 /// Check (isSamplerInitializer || isPipeStorageInitializer)
 bool isSpecialTypeInitializer(Instruction *Inst);
 
-bool isPipeBI(const StringRef MangledName);
+bool isPipeOrAddressSpaceCastBI(const StringRef MangledName);
 bool isEnqueueKernelBI(const StringRef MangledName);
 bool isKernelQueryBI(const StringRef MangledName);
 
@@ -747,6 +747,10 @@ template <> inline void SPIRVMap<std::string, Op, SPIRVInstruction>::init() {
   _SPIRV_OP(intel_sub_group_shuffle_down, SubgroupShuffleDownINTEL)
   _SPIRV_OP(intel_sub_group_shuffle_up, SubgroupShuffleUpINTEL)
   _SPIRV_OP(intel_sub_group_shuffle_xor, SubgroupShuffleXorINTEL)
+  // Intel media_block_io builtins
+  _SPIRV_OP(intel_sub_group_media_block_read, SubgroupImageMediaBlockReadINTEL)
+  _SPIRV_OP(intel_sub_group_media_block_write,
+            SubgroupImageMediaBlockWriteINTEL)
 #undef _SPIRV_OP
 }
 
@@ -760,6 +764,8 @@ template <> inline void SPIRVMap<std::string, Op, OCL12Builtin>::init() {
   _SPIRV_OP(dec, AtomicIDecrement)
   _SPIRV_OP(min, AtomicSMin)
   _SPIRV_OP(max, AtomicSMax)
+  _SPIRV_OP(umin, AtomicUMin)
+  _SPIRV_OP(umax, AtomicUMax)
   _SPIRV_OP(and, AtomicAnd)
   _SPIRV_OP(or, AtomicOr)
   _SPIRV_OP(xor, AtomicXor)
