@@ -79,7 +79,7 @@ public:
       // The source could be bit-cast from another type,
       // need the original type for the allocation of the temporary variable
       SrcTy = cast<BitCastInst>(Src)->getOperand(0)->getType();
-    auto Align = I.getSourceAlignment();
+    MaybeAlign Align = I.getSourceAlign();
     auto Volatile = I.isVolatile();
     Value *NumElements = nullptr;
     uint64_t ElementsCount = 1;
@@ -98,7 +98,7 @@ public:
     Alloca->setAlignment(Align);
     Builder.CreateLifetimeStart(Alloca);
     Builder.CreateMemCpy(Alloca, Align, Src, Align, Length, Volatile);
-    auto *SecondCpy = Builder.CreateMemCpy(Dest, I.getDestAlignment(), Alloca,
+    auto *SecondCpy = Builder.CreateMemCpy(Dest, I.getDestAlign(), Alloca,
                                            Align, Length, Volatile);
     Builder.CreateLifetimeEnd(Alloca);
 
