@@ -49,7 +49,7 @@
 namespace SPIRV {
 class SPIRVToOCL : public ModulePass, public InstVisitor<SPIRVToOCL> {
 protected:
-  SPIRVToOCL() : ModulePass(ID), M(nullptr), Ctx(nullptr) {}
+  SPIRVToOCL(char &ID) : ModulePass(ID), M(nullptr), Ctx(nullptr) {}
 
 public:
   virtual bool runOnModule(Module &M) = 0;
@@ -88,6 +88,12 @@ public:
   /// Transform __spirv_{PipeOpName} to OCL pipe builtin functions.
   void visitCallSPIRVPipeBuiltin(CallInst *CI, Op OC);
 
+  /// Transform __spirv_OpOpSubgroupImageMediaBlockReadINTEL =>
+  ///  intel_sub_group_media_block_read
+  ///           __spirv_OpSubgroupImageMediaBlockWriteINTEL =>
+  ///  intel_sub_group_media_block_write
+  void visitCallSPIRVImageMediaBlockBuiltin(CallInst *CI, Op OC);
+
   /// Transform __spirv_* builtins to OCL 2.0 builtins.
   /// No change with arguments.
   void visitCallSPIRVBuiltin(CallInst *CI, Op OC);
@@ -133,8 +139,6 @@ public:
   /// using separate maps for OpenCL 1.2 and OpenCL 2.0
   virtual Instruction *mutateAtomicName(CallInst *CI, Op OC) = 0;
 
-  static char ID;
-
 protected:
   Module *M;
   LLVMContext *Ctx;
