@@ -45,7 +45,7 @@ namespace SPIRV {
 
 class SPIRVToOCL12 : public SPIRVToOCL {
 public:
-  SPIRVToOCL12() {
+  SPIRVToOCL12() : SPIRVToOCL(ID) {
     initializeSPIRVToOCL12Pass(*PassRegistry::getPassRegistry());
   }
   bool runOnModule(Module &M) override;
@@ -103,8 +103,12 @@ public:
   /// types or with "atom_" for 64-bit types, as specified by
   /// cl_khr_int64_base_atomics and cl_khr_int64_extended_atomics extensions.
   std::string mapAtomicName(Op OC, Type *Ty);
+
+  static char ID;
 };
 
+char SPIRVToOCL12::ID = 0;
+
 bool SPIRVToOCL12::runOnModule(Module &Module) {
   M = &Module;
   Ctx = &M->getContext();
@@ -221,8 +225,7 @@ Instruction *SPIRVToOCL12::visitCallSPIRVAtomicUMinUMax(CallInst *CI, Op OC) {
       [=](CallInst *, std::vector<Value *> &Args) {
         std::swap(Args[1], Args[3]);
         Args.resize(2);
-        return mapAtomicName(OC == OpAtomicUMin ? OpAtomicSMin : OpAtomicSMax,
-                             CI->getType());
+        return mapAtomicName(OC, CI->getType());
       },
       &Attrs);
 }
