@@ -461,13 +461,14 @@ bool getSPIRVBuiltin(const std::string &OrigName, spv::BuiltIn &B) {
   return getByName(R.str(), B);
 }
 
-// Enqueue kernel, kernel query and pipe built-ins are not mangled
+// Enqueue kernel, kernel query, pipe and address space cast built-ins
+// are not mangled.
 bool isNonMangledOCLBuiltin(const StringRef &Name) {
   if (!Name.startswith("__"))
     return false;
 
   return isEnqueueKernelBI(Name) || isKernelQueryBI(Name) ||
-         isPipeBI(Name.drop_front(2));
+         isPipeOrAddressSpaceCastBI(Name.drop_front(2));
 }
 
 bool oclIsBuiltin(const StringRef &Name, std::string *DemangledName,
@@ -850,20 +851,23 @@ int getMDOperandAsInt(MDNode *N, unsigned I) {
   return mdconst::dyn_extract<ConstantInt>(N->getOperand(I))->getZExtValue();
 }
 
-std::string getMDOperandAsString(MDNode *N, unsigned I) {
+// Additional helper function to be reused by getMDOperandAs* helpers
+Metadata *getMDOperandOrNull(MDNode *N, unsigned I) {
   if (!N)
-    return "";
+    return nullptr;
+  return N->getOperand(I);
+}
 
-  Metadata *Op = N->getOperand(I);
-  if (!Op)
-    return "";
-
-  if (MDString *Str = dyn_cast<MDString>(Op)) {
+std::string getMDOperandAsString(MDNode *N, unsigned I) {
+  if (auto *Str = dyn_cast_or_null<MDString>(getMDOperandOrNull(N, I)))
     return Str->getString().str();
-  }
   return "";
 }
 
+MDNode *getMDOperandAsMDNode(MDNode *N, unsigned I) {
+  return dyn_cast_or_null<MDNode>(getMDOperandOrNull(N, I));
+}
+
 Type *getMDOperandAsType(MDNode *N, unsigned I) {
   return cast<ValueAsMetadata>(N->getOperand(I))->getType();
 }
@@ -1391,10 +1395,12 @@ bool eraseUselessFunctions(Module *M) {
 }
 
 // The mangling algorithm follows OpenCL pipe built-ins clang 3.8 CodeGen rules.
-static SPIR::MangleError manglePipeBuiltin(const SPIR::FunctionDescriptor &Fd,
-                                           std::string &MangledName) {
-  assert(OCLUtil::isPipeBI(Fd.Name) &&
-         "Method is expected to be called only for pipe builtins!");
+static SPIR::MangleError
+manglePipeOrAddressSpaceCastBuiltin(const SPIR::FunctionDescriptor &Fd,
+                                    std::string &MangledName) {
+  assert(OCLUtil::isPipeOrAddressSpaceCastBI(Fd.Name) &&
+         "Method is expected to be called only for pipe and address space cast "
+         "builtins!");
   if (Fd.isNull()) {
     MangledName.assign(SPIR::FunctionDescriptor::nullString());
     return SPIR::MANGLE_NULL_FUNC_DESCRIPTOR;
@@ -1443,8 +1449,8 @@ std::string mangleBuiltin(const std::string &UniqName,
   SPIR::NameMangler Mangler(SPIR::SPIR20);
   Mangler.mangle(FD, MangledName);
 #else
-  if (OCLUtil::isPipeBI(BtnInfo->getUnmangledName())) {
-    manglePipeBuiltin(FD, MangledName);
+  if (OCLUtil::isPipeOrAddressSpaceCastBI(BtnInfo->getUnmangledName())) {
+    manglePipeOrAddressSpaceCastBuiltin(FD, MangledName);
   } else {
     SPIR::NameMangler Mangler(SPIR::SPIR20);
     Mangler.mangle(FD, MangledName);
@@ -1498,72 +1504,14 @@ llvm::Constant *getOCLNullClkEventPtr(Module *M) {
   return Constant::getNullValue(getOCLClkEventPtrType(M));
 }
 
-bool hasLoopUnrollMetadata(const Module *M) {
+bool hasLoopMetadata(const Module *M) {
   for (const Function &F : *M)
     for (const BasicBlock &BB : F) {
       const Instruction *Term = BB.getTerminator();
-      if (!Term)
-        continue;
-      if (const MDNode *MD = Term->getMetadata("llvm.loop"))
-        for (const MDOperand &MDOp : MD->operands())
-          if (getMDOperandAsString(dyn_cast<MDNode>(MDOp), 0)
-                  .find("llvm.loop.unroll.") == 0)
-            return true;
+      if (Term && Term->getMetadata("llvm.loop"))
+        return true;
     }
   return false;
 }
 
-spv::LoopControlMask getLoopControl(const BranchInst *Branch,
-                                    std::vector<SPIRVWord> &Parameters) {
-  if (!Branch)
-    return spv::LoopControlMaskNone;
-  MDNode *LoopMD = Branch->getMetadata("llvm.loop");
-  if (!LoopMD)
-    return spv::LoopControlMaskNone;
-
-  size_t LoopControl = spv::LoopControlMaskNone;
-  for (const MDOperand &MDOp : LoopMD->operands()) {
-    if (MDNode *Node = dyn_cast<MDNode>(MDOp)) {
-      std::string S = getMDOperandAsString(Node, 0);
-      // Set the loop control bits. Parameters are set in the order described
-      // in 3.23 SPIR-V Spec. rev. 1.4:
-      // Bits that are set can indicate whether an additional operand follows,
-      // as described by the table. If there are multiple following operands
-      // indicated, they are ordered: Those indicated by smaller-numbered bits
-      // appear first.
-      if (S == "llvm.loop.unroll.disable")
-        LoopControl |= spv::LoopControlDontUnrollMask;
-      else if (S == "llvm.loop.unroll.full" || S == "llvm.loop.unroll.enable")
-        LoopControl |= spv::LoopControlUnrollMask;
-      if (S == "llvm.loop.ivdep.enable")
-        LoopControl |= spv::LoopControlDependencyInfiniteMask;
-      if (S == "llvm.loop.ivdep.safelen") {
-        size_t I = getMDOperandAsInt(Node, 1);
-        Parameters.push_back(I);
-        LoopControl |= spv::LoopControlDependencyLengthMask;
-      }
-      // PartialCount must not be used with the DontUnroll bit
-      if (S == "llvm.loop.unroll.count" &&
-          !(LoopControl & LoopControlDontUnrollMask)) {
-        size_t I = getMDOperandAsInt(Node, 1);
-        Parameters.push_back(I);
-        LoopControl |= spv::LoopControlPartialCountMask;
-      }
-      if (S == "llvm.loop.ii.count") {
-        Parameters.push_back(InitiationIntervalINTEL);
-        size_t I = getMDOperandAsInt(Node, 1);
-        Parameters.push_back(I);
-        LoopControl |= spv::LoopControlExtendedControlsMask;
-      }
-      if (S == "llvm.loop.max_concurrency.count") {
-        Parameters.push_back(MaxConcurrencyINTEL);
-        size_t I = getMDOperandAsInt(Node, 1);
-        Parameters.push_back(I);
-        LoopControl |= spv::LoopControlExtendedControlsMask;
-      }
-    }
-  }
-  return static_cast<spv::LoopControlMask>(LoopControl);
-}
-
 } // namespace SPIRV
