@@ -40,6 +40,7 @@
 
 #include "SPIRVWriter.h"
 #include "LLVMToSPIRVDbgTran.h"
+#include "SPIRVAsm.h"
 #include "SPIRVBasicBlock.h"
 #include "SPIRVEntry.h"
 #include "SPIRVEnum.h"
@@ -56,9 +57,11 @@
 #include "llvm/ADT/DenseMap.h"
 #include "llvm/ADT/StringSwitch.h"
 #include "llvm/ADT/Triple.h"
+#include "llvm/Analysis/ValueTracking.h"
 #include "llvm/IR/Constants.h"
 #include "llvm/IR/DerivedTypes.h"
 #include "llvm/IR/Function.h"
+#include "llvm/IR/InlineAsm.h"
 #include "llvm/IR/InstrTypes.h"
 #include "llvm/IR/Instructions.h"
 #include "llvm/IR/IntrinsicInst.h"
@@ -103,12 +106,12 @@ static void foreachKernelArgMD(
 LLVMToSPIRV::LLVMToSPIRV(SPIRVModule *SMod)
     : ModulePass(ID), M(nullptr), Ctx(nullptr), BM(SMod), SrcLang(0),
       SrcLangVer(0) {
-  DbgTran = make_unique<LLVMToSPIRVDbgTran>(nullptr, SMod, this);
+  DbgTran = std::make_unique<LLVMToSPIRVDbgTran>(nullptr, SMod, this);
 }
 
 bool LLVMToSPIRV::runOnModule(Module &Mod) {
   M = &Mod;
-  CG = make_unique<CallGraph>(Mod);
+  CG = std::make_unique<CallGraph>(Mod);
   Ctx = &M->getContext();
   DbgTran->setModule(M);
   assert(BM && "SPIR-V module not initialized");
@@ -223,8 +226,17 @@ static bool recursiveType(const StructType *ST, const Type *Ty) {
              StructTy->element_end();
     }
 
-    if (auto *PtrTy = dyn_cast<PointerType>(Ty))
-      return Run(PtrTy->getPointerElementType());
+    if (auto *PtrTy = dyn_cast<PointerType>(Ty)) {
+      Type *ElTy = PtrTy->getPointerElementType();
+      if (auto *FTy = dyn_cast<FunctionType>(ElTy)) {
+        // If we have a function pointer, then argument types and return type of
+        // the referenced function also need to be checked
+        return Run(FTy->getReturnType()) ||
+               any_of(FTy->param_begin(), FTy->param_end(), Run);
+      }
+
+      return Run(ElTy);
+    }
 
     if (auto *ArrayTy = dyn_cast<ArrayType>(Ty))
       return Run(ArrayTy->getArrayElementType());
@@ -247,8 +259,17 @@ SPIRVType *LLVMToSPIRV::transType(Type *T) {
   if (T->isIntegerTy(1))
     return mapType(T, BM->addBoolType());
 
-  if (T->isIntegerTy())
-    return mapType(T, BM->addIntegerType(T->getIntegerBitWidth()));
+  if (T->isIntegerTy()) {
+    unsigned BitWidth = T->getIntegerBitWidth();
+    // SPIR-V 2.16.1. Universal Validation Rules: Scalar integer types can be
+    // parameterized only as 32 bit, plus any additional sizes enabled by
+    // capabilities.
+    if (BM->getErrorLog().checkError(
+            BitWidth == 8 || BitWidth == 16 || BitWidth == 32 || BitWidth == 64,
+            SPIRVEC_InvalidBitWidth, std::to_string(BitWidth))) {
+      return mapType(T, BM->addIntegerType(T->getIntegerBitWidth()));
+    }
+  }
 
   if (T->isFloatingPointTy())
     return mapType(T, BM->addFloatType(T->getPrimitiveSizeInBits()));
@@ -654,10 +675,20 @@ SPIRVInstruction *LLVMToSPIRV::transBinaryInst(BinaryOperator *B,
 }
 
 SPIRVInstruction *LLVMToSPIRV::transCmpInst(CmpInst *Cmp, SPIRVBasicBlock *BB) {
-  auto Op0 = transValue(Cmp->getOperand(0), BB);
-  SPIRVInstruction *BI = BM->addCmpInst(
-      transBoolOpCode(Op0, CmpMap::map(Cmp->getPredicate())),
-      transType(Cmp->getType()), Op0, transValue(Cmp->getOperand(1), BB), BB);
+  auto *Op0 = Cmp->getOperand(0);
+  SPIRVValue *TOp0 = transValue(Op0, BB);
+  SPIRVValue *TOp1 = transValue(Cmp->getOperand(1), BB);
+  // TODO: once the translator supports SPIR-V 1.4, update the condition below:
+  // if (/* */->isPointerTy() && /* it is not allowed to use SPIR-V 1.4 */)
+  if (Op0->getType()->isPointerTy()) {
+    unsigned AS = cast<PointerType>(Op0->getType())->getAddressSpace();
+    SPIRVType *Ty = transType(getSizetType(AS));
+    TOp0 = BM->addUnaryInst(OpConvertPtrToU, Ty, TOp0, BB);
+    TOp1 = BM->addUnaryInst(OpConvertPtrToU, Ty, TOp1, BB);
+  }
+  SPIRVInstruction *BI =
+      BM->addCmpInst(transBoolOpCode(TOp0, CmpMap::map(Cmp->getPredicate())),
+                     transType(Cmp->getType()), TOp0, TOp1, BB);
   return BI;
 }
 
@@ -685,6 +716,153 @@ SPIRV::SPIRVInstruction *LLVMToSPIRV::transUnaryInst(UnaryInstruction *U,
                           BB);
 }
 
+/// This helper class encapsulates information extraction from
+/// "llvm.loop.parallel_access_indices" metadata hints. Initialize
+/// with a pointer to an MDNode with the following structure:
+/// !<Node> = !{!"llvm.loop.parallel_access_indices", !<Node>, !<Node>, ...}
+/// OR:
+/// !<Node> = !{!"llvm.loop.parallel_access_indices", !<Nodes...>, i32 <value>}
+///
+/// All of the MDNode-type operands mark the index groups for particular
+/// array variables. An optional i32 value indicates the safelen (safe
+/// number of iterations) for the optimization application to these
+/// array variables. If the safelen value is absent, an infinite
+/// number of iterations is implied.
+class LLVMParallelAccessIndices {
+public:
+  LLVMParallelAccessIndices(
+      MDNode *Node, LLVMToSPIRV::LLVMToSPIRVMetadataMap &IndexGroupArrayMap)
+      : Node(Node), IndexGroupArrayMap(IndexGroupArrayMap) {}
+
+  void initialize() {
+    assert(isValid() &&
+           "LLVMParallelAccessIndices initialized from an invalid MDNode");
+
+    unsigned NumOperands = Node->getNumOperands();
+    auto *SafeLenExpression = mdconst::dyn_extract_or_null<ConstantInt>(
+        Node->getOperand(NumOperands - 1));
+    // If no safelen value is specified and the last operand
+    // casts to an MDNode* rather than an int, 0 will be stored
+    SafeLen = SafeLenExpression ? SafeLenExpression->getZExtValue() : 0;
+
+    // Count MDNode operands that refer to index groups:
+    // - operand [0] is a string literal and should be ignored;
+    // - depending on whether a particular safelen is specified as the
+    //   last operand, we may or may not want to extract the latter
+    //   as an index group
+    unsigned NumIdxGroups = SafeLen ? NumOperands - 2 : NumOperands - 1;
+    for (unsigned I = 1; I <= NumIdxGroups; ++I) {
+      MDNode *IdxGroupNode = getMDOperandAsMDNode(Node, I);
+      assert(IdxGroupNode &&
+             "Invalid operand in the MDNode for LLVMParallelAccessIndices");
+      auto IdxGroupArrayPairIt = IndexGroupArrayMap.find(IdxGroupNode);
+      assert(IdxGroupArrayPairIt != IndexGroupArrayMap.end() &&
+             "Absent entry for this index group node");
+      ArrayVariablesVec.push_back(IdxGroupArrayPairIt->second);
+    }
+  }
+
+  bool isValid() {
+    bool IsNamedCorrectly = getMDOperandAsString(Node, 0) == ExpectedName;
+    return Node && IsNamedCorrectly;
+  }
+
+  unsigned getSafeLen() { return SafeLen; }
+  const std::vector<SPIRVId> &getArrayVariables() { return ArrayVariablesVec; }
+
+private:
+  MDNode *Node;
+  LLVMToSPIRV::LLVMToSPIRVMetadataMap &IndexGroupArrayMap;
+  const std::string ExpectedName = "llvm.loop.parallel_access_indices";
+  std::vector<SPIRVId> ArrayVariablesVec;
+  unsigned SafeLen;
+};
+
+/// Go through the operands !llvm.loop metadata attached to the branch
+/// instruction, fill the Loop Control mask and possible parameters for its
+/// fields.
+static spv::LoopControlMask
+getLoopControl(const BranchInst *Branch, std::vector<SPIRVWord> &Parameters,
+               LLVMToSPIRV::LLVMToSPIRVMetadataMap &IndexGroupArrayMap) {
+  if (!Branch)
+    return spv::LoopControlMaskNone;
+  MDNode *LoopMD = Branch->getMetadata("llvm.loop");
+  if (!LoopMD)
+    return spv::LoopControlMaskNone;
+
+  size_t LoopControl = spv::LoopControlMaskNone;
+
+  // Unlike with most of the cases, some loop metadata specifications
+  // can occur multiple times - for these, all correspondent tokens
+  // need to be collected first, and only then added to SPIR-V loop
+  // parameters in a separate routine
+  std::vector<std::pair<SPIRVWord, SPIRVWord>> DependencyArrayParameters;
+
+  for (const MDOperand &MDOp : LoopMD->operands()) {
+    if (MDNode *Node = dyn_cast<MDNode>(MDOp)) {
+      std::string S = getMDOperandAsString(Node, 0);
+      // Set the loop control bits. Parameters are set in the order described
+      // in 3.23 SPIR-V Spec. rev. 1.4:
+      // Bits that are set can indicate whether an additional operand follows,
+      // as described by the table. If there are multiple following operands
+      // indicated, they are ordered: Those indicated by smaller-numbered bits
+      // appear first.
+      if (S == "llvm.loop.unroll.disable")
+        LoopControl |= spv::LoopControlDontUnrollMask;
+      else if (S == "llvm.loop.unroll.full" || S == "llvm.loop.unroll.enable")
+        LoopControl |= spv::LoopControlUnrollMask;
+      // PartialCount must not be used with the DontUnroll bit
+      else if (S == "llvm.loop.unroll.count" &&
+               !(LoopControl & LoopControlDontUnrollMask)) {
+        size_t I = getMDOperandAsInt(Node, 1);
+        Parameters.push_back(I);
+        LoopControl |= spv::LoopControlPartialCountMask;
+      } else if (S == "llvm.loop.ivdep.enable")
+        LoopControl |= spv::LoopControlDependencyInfiniteMask;
+      else if (S == "llvm.loop.ivdep.safelen") {
+        size_t I = getMDOperandAsInt(Node, 1);
+        Parameters.push_back(I);
+        LoopControl |= spv::LoopControlDependencyLengthMask;
+      } else if (S == "llvm.loop.ii.count") {
+        size_t I = getMDOperandAsInt(Node, 1);
+        Parameters.push_back(I);
+        LoopControl |= spv::InitiationIntervalINTEL;
+      } else if (S == "llvm.loop.max_concurrency.count") {
+        size_t I = getMDOperandAsInt(Node, 1);
+        Parameters.push_back(I);
+        LoopControl |= spv::MaxConcurrencyINTEL;
+      } else if (S == "llvm.loop.parallel_access_indices") {
+        // Intel FPGA IVDep loop attribute
+        LLVMParallelAccessIndices IVDep(Node, IndexGroupArrayMap);
+        IVDep.initialize();
+        // Store IVDep-specific parameters into an intermediate
+        // container to address the case when there're multiple
+        // IVDep metadata nodes and this condition gets entered multiple
+        // times. The update of the main parameters vector & the loop control
+        // mask will be done later, in the main scope of the function
+        unsigned SafeLen = IVDep.getSafeLen();
+        for (auto &ArrayId : IVDep.getArrayVariables())
+          DependencyArrayParameters.emplace_back(ArrayId, SafeLen);
+      }
+    }
+  }
+
+  // If any loop control parameters were held back until fully collected,
+  // now is the time to move the information to the main parameters collection
+  if (!DependencyArrayParameters.empty()) {
+    // The first parameter states the number of <array, safelen> pairs to be
+    // listed
+    Parameters.push_back(DependencyArrayParameters.size());
+    for (auto &ArraySflnPair : DependencyArrayParameters) {
+      Parameters.push_back(ArraySflnPair.first);
+      Parameters.push_back(ArraySflnPair.second);
+    }
+    LoopControl |= spv::DependencyArrayINTEL;
+  }
+
+  return static_cast<spv::LoopControlMask>(LoopControl);
+}
+
 /// An instruction may use an instruction from another BB which has not been
 /// translated. SPIRVForward should be created as place holder for these
 /// instructions and replaced later by the real instructions.
@@ -890,7 +1068,8 @@ SPIRVValue *LLVMToSPIRV::transValueWithoutDecoration(Value *V,
     /// with true edge going to the header and the false edge going out of
     /// the loop, which corresponds to a "Merge Block" per the SPIR-V spec.
     std::vector<SPIRVWord> Parameters;
-    spv::LoopControlMask LoopControl = getLoopControl(Branch, Parameters);
+    spv::LoopControlMask LoopControl =
+        getLoopControl(Branch, Parameters, IndexGroupArrayMap);
 
     if (Branch->isUnconditional()) {
       // For "for" and "while" loops llvm.loop metadata is attached to
@@ -975,10 +1154,36 @@ SPIRVValue *LLVMToSPIRV::transValueWithoutDecoration(Value *V,
     std::vector<SPIRVValue *> Indices;
     for (unsigned I = 0, E = GEP->getNumIndices(); I != E; ++I)
       Indices.push_back(transValue(GEP->getOperand(I + 1), BB));
-    return mapValue(
-        V, BM->addPtrAccessChainInst(transType(GEP->getType()),
-                                     transValue(GEP->getPointerOperand(), BB),
-                                     Indices, BB, GEP->isInBounds()));
+    auto *TransPointerOperand = transValue(GEP->getPointerOperand(), BB);
+
+    // Certain array-related optimization hints can be expressed via
+    // LLVM metadata. For the purpose of linking this metadata with
+    // the accessed array variables, our GEP may have been marked into
+    // a so-called index group, an MDNode by itself.
+    if (MDNode *IndexGroup = GEP->getMetadata("llvm.index.group")) {
+      // When where we work with embedded loops, it's natural that
+      // the outer loop's hints apply to all code contained within.
+      // The inner loop's specific hints, however, should stay private
+      // to the inner loop's scope.
+      // Consequently, the following division of the index group metadata
+      // nodes emerges:
+      // 1) The metadata node has no operands. It will be directly referenced
+      //    from within the optimization hint metadata.
+      // 2) The metadata node has several operands. It serves to link an index
+      //    group specific to some embedded loop with other index groups that
+      //    mark the same array variable for the outer loop(s).
+      unsigned NumOperands = IndexGroup->getNumOperands();
+      if (NumOperands > 0)
+        // The index group for this particular "embedded loop depth" is always
+        // signalled by the last variable. We'll want to associate this loop's
+        // control parameters with this inner-loop-specific index group
+        IndexGroup = getMDOperandAsMDNode(IndexGroup, NumOperands - 1);
+      IndexGroupArrayMap[IndexGroup] = TransPointerOperand->getId();
+    }
+
+    return mapValue(V, BM->addPtrAccessChainInst(transType(GEP->getType()),
+                                                 TransPointerOperand, Indices,
+                                                 BB, GEP->isInBounds()));
   }
 
   if (auto Ext = dyn_cast<ExtractElementInst>(V)) {
@@ -1025,6 +1230,10 @@ SPIRVValue *LLVMToSPIRV::transValueWithoutDecoration(Value *V,
     return BV ? mapValue(V, BV) : nullptr;
   }
 
+  if (InlineAsm *IA = dyn_cast<InlineAsm>(V))
+    if (BM->isAllowedToUseExtension(ExtensionID::SPV_INTEL_inline_assembly))
+      return mapValue(V, transAsmINTEL(IA));
+
   if (CallInst *CI = dyn_cast<CallInst>(V))
     return mapValue(V, transCallInst(CI, BB));
 
@@ -1174,8 +1383,9 @@ tryParseIntelFPGAAnnotationString(StringRef AnnotatedCode) {
                 .Case("memory", DecorationMemoryINTEL)
                 .Case("numbanks", DecorationNumbanksINTEL)
                 .Case("bankwidth", DecorationBankwidthINTEL)
-                .Case("max_private_copies", DecorationMaxPrivateCopiesINTEL)
+                .Case("private_copies", DecorationMaxPrivateCopiesINTEL)
                 .Case("max_replicates", DecorationMaxReplicatesINTEL)
+                .Case("bank_bits", DecorationBankBitsINTEL)
                 .Case("merge", DecorationMergeINTEL)
                 .Default(DecorationUserSemantic);
       if (Dec == DecorationUserSemantic)
@@ -1190,6 +1400,18 @@ tryParseIntelFPGAAnnotationString(StringRef AnnotatedCode) {
   return Decorates;
 }
 
+std::vector<SPIRVWord> getBankBitsFromString(StringRef S) {
+  SmallVector<StringRef, 4> BitsString;
+  S.split(BitsString, ',');
+
+  std::vector<SPIRVWord> Bits(BitsString.size());
+  for (size_t J = 0; J < BitsString.size(); ++J)
+    if (BitsString[J].getAsInteger(10, Bits[J]))
+      return {};
+
+  return Bits;
+}
+
 void addIntelFPGADecorations(
     SPIRVEntry *E,
     std::vector<std::pair<Decoration, std::string>> &Decorations) {
@@ -1198,6 +1420,11 @@ void addIntelFPGADecorations(
     return;
 
   for (const auto &I : Decorations) {
+    // Such decoration already exists on a type, skip it
+    if (E->hasDecorate(I.first, /*Index=*/0, /*Result=*/nullptr)) {
+      continue;
+    }
+
     switch (I.first) {
     case DecorationUserSemantic:
       E->addDecorate(new SPIRVDecorateUserSemanticAttr(E, I.second));
@@ -1211,6 +1438,10 @@ void addIntelFPGADecorations(
       E->addDecorate(
           new SPIRVDecorateMergeINTELAttr(E, Name.str(), Direction.str()));
     } break;
+    case DecorationBankBitsINTEL:
+      E->addDecorate(new SPIRVDecorateBankBitsINTELAttr(
+          E, getBankBitsFromString(I.second)));
+      break;
     case DecorationRegisterINTEL:
     case DecorationSinglepumpINTEL:
     case DecorationDoublepumpINTEL:
@@ -1240,6 +1471,12 @@ void addIntelFPGADecorationsForStructMember(
     return;
 
   for (const auto &I : Decorations) {
+    // Such decoration already exists on a type, skip it
+    if (E->hasMemberDecorate(I.first, /*Index=*/0, MemberNumber,
+                             /*Result=*/nullptr)) {
+      continue;
+    }
+
     switch (I.first) {
     case DecorationUserSemantic:
       E->addMemberDecorate(
@@ -1255,6 +1492,10 @@ void addIntelFPGADecorationsForStructMember(
       E->addMemberDecorate(new SPIRVMemberDecorateMergeINTELAttr(
           E, MemberNumber, Name.str(), Direction.str()));
     } break;
+    case DecorationBankBitsINTEL:
+      E->addMemberDecorate(new SPIRVMemberDecorateBankBitsINTELAttr(
+          E, MemberNumber, getBankBitsFromString(I.second)));
+      break;
     case DecorationRegisterINTEL:
     case DecorationSinglepumpINTEL:
     case DecorationDoublepumpINTEL:
@@ -1304,6 +1545,15 @@ SPIRVValue *LLVMToSPIRV::transIntrinsicInst(IntrinsicInst *II,
     SPIRVValue *Op = transValue(II->getArgOperand(0), BB);
     return BM->addUnaryInst(OpBitReverse, Ty, Op, BB);
   }
+  case Intrinsic::ctlz:
+  case Intrinsic::cttz: {
+    SPIRVWord ExtOp = II->getIntrinsicID() == Intrinsic::ctlz ? OpenCLLIB::Clz
+                                                              : OpenCLLIB::Ctz;
+    SPIRVType *Ty = transType(II->getType());
+    std::vector<SPIRVValue *> Ops(1, transValue(II->getArgOperand(0), BB));
+    return BM->addExtInst(Ty, BM->getExtInstSetId(SPIRVEIS_OpenCL), ExtOp, Ops,
+                          BB);
+  }
   case Intrinsic::fmuladd: {
     // For llvm.fmuladd.* fusion is not guaranteed. If a fused multiply-add
     // is required the corresponding llvm.fma.* intrinsic function should be
@@ -1380,9 +1630,8 @@ SPIRVValue *LLVMToSPIRV::transIntrinsicInst(IntrinsicInst *II,
     if (!GEP)
       return nullptr;
     Constant *C = cast<Constant>(GEP->getOperand(0));
-    // TODO: Refactor to use getConstantStringInfo()
-    StringRef AnnotationString =
-        cast<ConstantDataArray>(C->getOperand(0))->getAsCString();
+    StringRef AnnotationString;
+    getConstantStringInfo(C, AnnotationString);
 
     if (AnnotationString == kOCLBuiltinName::FPGARegIntel) {
       if (BM->isAllowedToUseExtension(ExtensionID::SPV_INTEL_fpga_reg))
@@ -1404,9 +1653,8 @@ SPIRVValue *LLVMToSPIRV::transIntrinsicInst(IntrinsicInst *II,
 
     GetElementPtrInst *GEP = cast<GetElementPtrInst>(II->getArgOperand(1));
     Constant *C = cast<Constant>(GEP->getOperand(0));
-    // TODO: Refactor to use getConstantStringInfo()
-    StringRef AnnotationString =
-        cast<ConstantDataArray>(C->getOperand(0))->getAsString();
+    StringRef AnnotationString;
+    getConstantStringInfo(C, AnnotationString);
 
     std::vector<std::pair<Decoration, std::string>> Decorations;
     if (BB->getModule()->isAllowedToUseExtension(
@@ -1418,28 +1666,27 @@ SPIRVValue *LLVMToSPIRV::transIntrinsicInst(IntrinsicInst *II,
     // If we didn't find any IntelFPGA-specific decorations, let's add the whole
     // annotation string as UserSemantic Decoration
     if (Decorations.empty()) {
-      SV->addDecorate(new SPIRVDecorateUserSemanticAttr(
-          SV, AnnotationString.substr(0, AnnotationString.size() - 1)));
+      SV->addDecorate(new SPIRVDecorateUserSemanticAttr(SV, AnnotationString));
     } else {
       addIntelFPGADecorations(SV, Decorations);
     }
     return SV;
   }
   case Intrinsic::ptr_annotation: {
-    GetElementPtrInst *GI;
-    if (auto *BI = dyn_cast<BitCastInst>(II->getArgOperand(0))) {
-      GI = dyn_cast<GetElementPtrInst>(BI->getOperand(0));
-    } else {
-      GI = dyn_cast<GetElementPtrInst>(II->getOperand(0));
-    }
-
     GetElementPtrInst *GEP = dyn_cast<GetElementPtrInst>(II->getArgOperand(1));
     Constant *C = dyn_cast<Constant>(GEP->getOperand(0));
-    // TODO: Refactor to use getConstantStringInfo()
-    StringRef AnnotationString =
-        dyn_cast<ConstantDataArray>(C->getOperand(0))->getAsCString();
+    StringRef AnnotationString;
+    getConstantStringInfo(C, AnnotationString);
 
-    if (GI) {
+    // Strip all bitcast and addrspace casts from the pointer argument:
+    //   llvm annotation intrinsic only takes i8*, so the original pointer
+    //   probably had to loose its addrspace and its original type.
+    Value *AnnotSubj = II->getArgOperand(0);
+    while (isa<BitCastInst>(AnnotSubj) || isa<AddrSpaceCastInst>(AnnotSubj)) {
+      AnnotSubj = cast<CastInst>(AnnotSubj)->getOperand(0);
+    }
+    // If the pointer is a GEP, then we have to emit a member decoration
+    if (auto *GI = dyn_cast<GetElementPtrInst>(AnnotSubj)) {
       auto *Ty = transType(GI->getSourceElementType());
       SPIRVWord MemberNumber =
           dyn_cast<ConstantInt>(GI->getOperand(2))->getZExtValue();
@@ -1488,6 +1735,11 @@ SPIRVValue *LLVMToSPIRV::transIntrinsicInst(IntrinsicInst *II,
 }
 
 SPIRVValue *LLVMToSPIRV::transCallInst(CallInst *CI, SPIRVBasicBlock *BB) {
+  assert(CI);
+  if (isa<InlineAsm>(CI->getCalledOperand()) &&
+      BM->isAllowedToUseExtension(ExtensionID::SPV_INTEL_inline_assembly))
+    return transAsmCallINTEL(CI, BB);
+
   if (CI->isIndirectCall())
     return transIndirectCallInst(CI, BB);
   return transDirectCallInst(CI, BB);
@@ -1538,6 +1790,32 @@ SPIRVValue *LLVMToSPIRV::transIndirectCallInst(CallInst *CI,
       BB);
 }
 
+SPIRVValue *LLVMToSPIRV::transAsmINTEL(InlineAsm *IA) {
+  assert(IA);
+
+  // TODO: intention here is to provide information about actual target
+  //       but in fact spir-64 is substituted as triple when translator works
+  //       eventually we need to fix it (not urgent)
+  StringRef TripleStr(M->getTargetTriple());
+  auto AsmTarget = static_cast<SPIRVAsmTargetINTEL *>(
+      BM->getOrAddAsmTargetINTEL(TripleStr.str()));
+  auto SIA = BM->addAsmINTEL(
+      static_cast<SPIRVTypeFunction *>(transType(IA->getFunctionType())),
+      AsmTarget, IA->getAsmString(), IA->getConstraintString());
+  if (IA->hasSideEffects())
+    SIA->addDecorate(DecorationSideEffectsINTEL);
+  return SIA;
+}
+
+SPIRVValue *LLVMToSPIRV::transAsmCallINTEL(CallInst *CI, SPIRVBasicBlock *BB) {
+  assert(CI);
+  auto IA = cast<InlineAsm>(CI->getCalledOperand());
+  return BM->addAsmCallINTELInst(
+      static_cast<SPIRVAsmINTEL *>(transValue(IA, BB, false)),
+      transArguments(CI, BB, SPIRVEntry::createUnique(OpAsmCallINTEL).get()),
+      BB);
+}
+
 bool LLVMToSPIRV::transAddressingMode() {
   Triple TargetTriple(M->getTargetTriple());
 
@@ -1585,9 +1863,45 @@ SPIRVWord LLVMToSPIRV::transFunctionControlMask(Function *F) {
   return FCM;
 }
 
+void LLVMToSPIRV::transGlobalAnnotation(GlobalVariable *V) {
+  SPIRVDBG(dbgs() << "[transGlobalAnnotation] " << *V << '\n');
+
+  // @llvm.global.annotations is an array that contains structs with 4 fields.
+  // Get the array of structs with metadata
+  ConstantArray *CA = cast<ConstantArray>(V->getOperand(0));
+  for (Value *Op : CA->operands()) {
+    ConstantStruct *CS = cast<ConstantStruct>(Op);
+    // The first field of the struct contains a pointer to annotated variable
+    Value *AnnotatedVar = CS->getOperand(0)->stripPointerCasts();
+    SPIRVValue *SV = transValue(AnnotatedVar, nullptr);
+
+    // The second field contains a pointer to a global annotation string
+    GlobalVariable *GV =
+        cast<GlobalVariable>(CS->getOperand(1)->stripPointerCasts());
+
+    StringRef AnnotationString;
+    getConstantStringInfo(GV, AnnotationString);
+
+    std::vector<std::pair<Decoration, std::string>> Decorations;
+    if (BM->isAllowedToUseExtension(
+            ExtensionID::SPV_INTEL_fpga_memory_attributes))
+      Decorations = tryParseIntelFPGAAnnotationString(AnnotationString);
+
+    // If we didn't find any IntelFPGA-specific decorations, let's
+    // add the whole annotation string as UserSemantic Decoration
+    if (Decorations.empty()) {
+      SV->addDecorate(new SPIRVDecorateUserSemanticAttr(SV, AnnotationString));
+    } else {
+      addIntelFPGADecorations(SV, Decorations);
+    }
+  }
+}
+
 bool LLVMToSPIRV::transGlobalVariables() {
   for (auto I = M->global_begin(), E = M->global_end(); I != E; ++I) {
-    if (!transValue(&(*I), nullptr))
+    if ((*I).getName() == "llvm.global.annotations")
+      transGlobalAnnotation(&(*I));
+    else if (!transValue(&(*I), nullptr))
       return false;
   }
   return true;
@@ -1744,9 +2058,9 @@ bool LLVMToSPIRV::translate() {
   return true;
 }
 
-llvm::IntegerType *LLVMToSPIRV::getSizetType() {
+llvm::IntegerType *LLVMToSPIRV::getSizetType(unsigned AS) {
   return IntegerType::getIntNTy(M->getContext(),
-                                M->getDataLayout().getPointerSizeInBits());
+                                M->getDataLayout().getPointerSizeInBits(AS));
 }
 
 void LLVMToSPIRV::oclGetMutatedArgumentTypesByBuiltin(
@@ -1808,6 +2122,16 @@ bool LLVMToSPIRV::transExecutionMode() {
         BF->addExecutionMode(BM->add(new SPIRVExecutionMode(
             BF, static_cast<ExecutionMode>(EMode), X, Y, Z)));
       } break;
+      case spv::ExecutionModeMaxWorkgroupSizeINTEL: {
+        if (BM->isAllowedToUseExtension(
+                ExtensionID::SPV_INTEL_kernel_attributes)) {
+          unsigned X, Y, Z;
+          N.get(X).get(Y).get(Z);
+          BF->addExecutionMode(BM->add(new SPIRVExecutionMode(
+              BF, static_cast<ExecutionMode>(EMode), X, Y, Z)));
+          BM->addCapability(CapabilityKernelAttributesINTEL);
+        }
+      } break;
       case spv::ExecutionModeVecTypeHint:
       case spv::ExecutionModeSubgroupSize:
       case spv::ExecutionModeSubgroupsPerWorkgroup: {
@@ -1816,6 +2140,26 @@ bool LLVMToSPIRV::transExecutionMode() {
         BF->addExecutionMode(BM->add(
             new SPIRVExecutionMode(BF, static_cast<ExecutionMode>(EMode), X)));
       } break;
+      case spv::ExecutionModeNumSIMDWorkitemsINTEL: {
+        if (BM->isAllowedToUseExtension(
+                ExtensionID::SPV_INTEL_kernel_attributes)) {
+          unsigned X;
+          N.get(X);
+          BF->addExecutionMode(BM->add(new SPIRVExecutionMode(
+              BF, static_cast<ExecutionMode>(EMode), X)));
+          BM->addCapability(CapabilityFPGAKernelAttributesINTEL);
+        }
+      } break;
+      case spv::ExecutionModeMaxWorkDimINTEL: {
+        if (BM->isAllowedToUseExtension(
+                ExtensionID::SPV_INTEL_kernel_attributes)) {
+          unsigned X;
+          N.get(X);
+          BF->addExecutionMode(BM->add(new SPIRVExecutionMode(
+              BF, static_cast<ExecutionMode>(EMode), X)));
+          BM->addCapability(CapabilityFPGAKernelAttributesINTEL);
+        }
+      } break;
       default:
         llvm_unreachable("invalid execution mode");
       }
@@ -2081,7 +2425,9 @@ bool llvm::writeSpirv(Module *M, const SPIRV::TranslatorOpts &Opts,
 
   legacy::PassManager PassMgr;
   addPassesForSPIRV(PassMgr);
-  if (hasLoopUnrollMetadata(M))
+  // Run loop simplify pass in order to avoid duplicate OpLoopMerge
+  // instruction. It can happen in case of continue operand in the loop.
+  if (hasLoopMetadata(M))
     PassMgr.add(createLoopSimplifyPass());
   PassMgr.add(createLLVMToSPIRV(BM.get()));
   PassMgr.run(*M);
