@@ -140,6 +140,9 @@ inline bool isValid(spv::ExecutionMode V) {
   case ExecutionModeFinalizer:
   case ExecutionModeSubgroupSize:
   case ExecutionModeSubgroupsPerWorkgroup:
+  case ExecutionModeMaxWorkgroupSizeINTEL:
+  case ExecutionModeMaxWorkDimINTEL:
+  case ExecutionModeNumSIMDWorkitemsINTEL:
     return true;
   default:
     return false;
@@ -405,6 +408,7 @@ inline bool isValid(spv::Decoration V) {
   case DecorationMaxPrivateCopiesINTEL:
   case DecorationSinglepumpINTEL:
   case DecorationDoublepumpINTEL:
+  case DecorationBankBitsINTEL:
   case DecorationReferencedIndirectlyINTEL:
     return true;
   default:
@@ -561,6 +565,8 @@ inline bool isValid(spv::Capability V) {
   case CapabilityFPGALoopControlsINTEL:
   case CapabilityBlockingPipesINTEL:
   case CapabilityUnstructuredLoopControlsINTEL:
+  case CapabilityKernelAttributesINTEL:
+  case CapabilityFPGAKernelAttributesINTEL:
     return true;
   default:
     return false;
@@ -886,6 +892,9 @@ inline bool isValid(spv::Op V) {
   case OpSubgroupImageBlockWriteINTEL:
   case OpSubgroupImageMediaBlockReadINTEL:
   case OpSubgroupImageMediaBlockWriteINTEL:
+  case OpAsmTargetINTEL:
+  case OpAsmINTEL:
+  case OpAsmCallINTEL:
   case OpVmeImageINTEL:
   case OpTypeVmeImageINTEL:
   case OpTypeAvcImePayloadINTEL:
@@ -1052,7 +1061,9 @@ inline bool isValidLoopControlMask(SPIRVWord Mask) {
   ValidMask |= LoopControlPartialCountMask;
   ValidMask |= LoopControlDependencyInfiniteMask;
   ValidMask |= LoopControlDependencyLengthMask;
-  ValidMask |= LoopControlExtendedControlsMask;
+  ValidMask |= InitiationIntervalINTEL;
+  ValidMask |= MaxConcurrencyINTEL;
+  ValidMask |= DependencyArrayINTEL;
 
   return (Mask & ~ValidMask) == 0;
 }
