@@ -135,6 +135,11 @@ inline bool isGroupOpCode(Op OpCode) {
   return OpGroupAll <= OC && OC <= OpGroupSMax;
 }
 
+inline bool isMediaBlockINTELOpcode(Op OpCode) {
+  return OpCode == OpSubgroupImageMediaBlockReadINTEL ||
+         OpCode == OpSubgroupImageMediaBlockWriteINTEL;
+}
+
 inline bool isPipeOpCode(Op OpCode) {
   unsigned OC = OpCode;
   return (OpReadPipe <= OC && OC <= OpGroupCommitWritePipe) ||
@@ -183,6 +188,7 @@ inline bool isIntelSubgroupOpCode(Op OpCode) {
   unsigned OC = OpCode;
   return OpSubgroupShuffleINTEL <= OC && OC <= OpSubgroupImageBlockWriteINTEL;
 }
+
 } // namespace SPIRV
 
 #endif // SPIRV_LIBSPIRV_SPIRVOPCODE_H
