@@ -136,34 +136,34 @@ protected:
   SPIRVType *Type; // Value Type
 };
 
-class SPIRVConstant : public SPIRVValue {
+template <spv::Op OC> class SPIRVConstantBase : public SPIRVValue {
 public:
   // Complete constructor for integer constant
-  SPIRVConstant(SPIRVModule *M, SPIRVType *TheType, SPIRVId TheId,
-                uint64_t TheValue)
-      : SPIRVValue(M, 0, OpConstant, TheType, TheId) {
+  SPIRVConstantBase(SPIRVModule *M, SPIRVType *TheType, SPIRVId TheId,
+                    uint64_t TheValue)
+      : SPIRVValue(M, 0, OC, TheType, TheId) {
     Union.UInt64Val = TheValue;
     recalculateWordCount();
     validate();
   }
   // Complete constructor for float constant
-  SPIRVConstant(SPIRVModule *M, SPIRVType *TheType, SPIRVId TheId,
-                float TheValue)
-      : SPIRVValue(M, 0, OpConstant, TheType, TheId) {
+  SPIRVConstantBase(SPIRVModule *M, SPIRVType *TheType, SPIRVId TheId,
+                    float TheValue)
+      : SPIRVValue(M, 0, OC, TheType, TheId) {
     Union.FloatVal = TheValue;
     recalculateWordCount();
     validate();
   }
   // Complete constructor for double constant
-  SPIRVConstant(SPIRVModule *M, SPIRVType *TheType, SPIRVId TheId,
-                double TheValue)
-      : SPIRVValue(M, 0, OpConstant, TheType, TheId) {
+  SPIRVConstantBase(SPIRVModule *M, SPIRVType *TheType, SPIRVId TheId,
+                    double TheValue)
+      : SPIRVValue(M, 0, OC, TheType, TheId) {
     Union.DoubleVal = TheValue;
     recalculateWordCount();
     validate();
   }
   // Incomplete constructor
-  SPIRVConstant() : SPIRVValue(OpConstant), NumWords(0) {}
+  SPIRVConstantBase() : SPIRVValue(OC), NumWords(0) {}
   uint64_t getZExtIntValue() const { return Union.UInt64Val; }
   float getFloatValue() const { return Union.FloatVal; }
   double getDoubleValue() const { return Union.DoubleVal; }
@@ -204,6 +204,9 @@ protected:
   } Union;
 };
 
+using SPIRVConstant = SPIRVConstantBase<OpConstant>;
+using SPIRVSpecConstant = SPIRVConstantBase<OpSpecConstant>;
+
 template <Op OC> class SPIRVConstantEmpty : public SPIRVValue {
 public:
   // Complete constructor
@@ -236,6 +239,8 @@ protected:
 
 typedef SPIRVConstantBool<OpConstantTrue> SPIRVConstantTrue;
 typedef SPIRVConstantBool<OpConstantFalse> SPIRVConstantFalse;
+typedef SPIRVConstantBool<OpSpecConstantTrue> SPIRVSpecConstantTrue;
+typedef SPIRVConstantBool<OpSpecConstantFalse> SPIRVSpecConstantFalse;
 
 class SPIRVConstantNull : public SPIRVConstantEmpty<OpConstantNull> {
 public:
@@ -273,18 +278,18 @@ protected:
   void validate() const override { SPIRVConstantEmpty::validate(); }
 };
 
-class SPIRVConstantComposite : public SPIRVValue {
+template <spv::Op OC> class SPIRVConstantCompositeBase : public SPIRVValue {
 public:
   // Complete constructor for composite constant
-  SPIRVConstantComposite(SPIRVModule *M, SPIRVType *TheType, SPIRVId TheId,
-                         const std::vector<SPIRVValue *> TheElements)
+  SPIRVConstantCompositeBase(SPIRVModule *M, SPIRVType *TheType, SPIRVId TheId,
+                             const std::vector<SPIRVValue *> TheElements)
       : SPIRVValue(M, TheElements.size() + 3, OpConstantComposite, TheType,
                    TheId) {
     Elements = getIds(TheElements);
     validate();
   }
   // Incomplete constructor
-  SPIRVConstantComposite() : SPIRVValue(OpConstantComposite) {}
+  SPIRVConstantCompositeBase() : SPIRVValue(OpConstantComposite) {}
   std::vector<SPIRVValue *> getElements() const { return getValues(Elements); }
   std::vector<SPIRVEntry *> getNonLiteralOperands() const override {
     std::vector<SPIRVValue *> Elements = getElements();
@@ -306,6 +311,10 @@ protected:
   std::vector<SPIRVId> Elements;
 };
 
+using SPIRVConstantComposite = SPIRVConstantCompositeBase<OpConstantComposite>;
+using SPIRVSpecConstantComposite =
+    SPIRVConstantCompositeBase<OpSpecConstantComposite>;
+
 class SPIRVConstantSampler : public SPIRVValue {
 public:
   const static Op OC = OpConstantSampler;
